/**
 * @author Tomasz Waleń (walen@mimuw.edu.pl)
 * @brief Rozwiazanie zadania Odrzucanie
 *        https://moodle.mimuw.edu.pl/mod/assign/view.php?id=77177
 * @date 2022-01-25
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "drzewo.h"

/**
 * @brief Wygneruj nowe drzewo, które zawiera te klucze z poddrzewa d, które mieszczą
 *        się w zakresie [a..b] 
 * 
 * @param d korzeń drzewa BST
 * @param a (int) dolna granica zakresu
 * @param b (int) górna granicza zakresu
 * @return Twezel* korzeń nowo utworzonego drzewa (ale może być również równy d)
 */
Twezel *pom(Twezel *d, int a, int b) {
    if (d==NULL) 
        return d;
    else {
        if (d->w < a) { // możemy zapomnieć o całym lewym podrzewie i wartości z d
            Twezel *tmp = d->psyn;
            usun(d->lsyn);
            free(d);
            return pom(tmp, a, b);
        } else if (b < d->w) { // możemy zapomnieć o całym prawym podrzewie i wartości z d
            Twezel *tmp = d->lsyn;
            usun(d->psyn);
            free(d);
            return pom(tmp, a, b);
        } else {
            // wartość z korzania jest w zakres [a..b]
            assert(a <= d->w && d->w <= b);
            d->lsyn = pom(d->lsyn, a, b);
            d->psyn = pom(d->psyn, a, b);
            return d;
        }
    }
}

void odrzuc(Twezel **d, int a, int b) {
    *d = pom(*d, a, b);
}

int main(void) {
    Twezel *d = czytaj();

#if defined(DOT)
    prolog_dot();
    pokaz_dot(d, "przed");
#elif defined(JSON)
    prolog_json();
    pokaz_json(d, "przed");
#else
    printf("przed:\n");
    pisz(d);
#endif

    odrzuc(&d, -100, 100);

#if defined(DOT)
    interlog_dot();
    pokaz_dot(d, "po");
    epilog_dot();
#elif defined(JSON)
    interlog_json();
    pokaz_json(d, "po");
    epilog_json();
#else
    printf("\n");
    printf("po: ");
    pisz(d);
#endif

    usun(d);
    return 0;
}
